; ---------------------------------------------------------
;	BIOS for H8/3048F
; =========================================================
;	(C)2004 t.hara
; ---------------------------------------------------------

	.cpu	300HA

; ---------------------------------------------------------
;	CPU registers
ABWCR		.equ		H'FFFFEC			; area bus width control register
BRCR		.equ		H'FFFFF3			; bus release controller enable register

; ---------------------------------------------------------
LcdDir		.equ		H'FFFFD1			; LCD port data direction register
LcdData		.equ		H'FFFFD3			; LCD port data register

LcdEN		.bequ		5,LcdData			; LCD signal E
LcdRS		.bequ		4,LcdData			; LCD signal RS

SMR0		.equ		H'FFFFB0
BRR0		.equ	 	H'FFFFB1
SCR0		.equ	 	H'FFFFB2
TDR0	 	.equ		H'FFFFB3
SSR0	 	.equ		H'FFFFB4
RDR0		.equ		H'FFFFB5
TE0			.bequ		5,SCR0
RE0			.bequ		6,SCR0
TDRE0		.bequ		7,SSR0
RDRF0		.bequ		6,SSR0

SMR1		.equ		H'FFFFB8
BRR1		.equ	 	H'FFFFB9
SCR1		.equ	 	H'FFFFBA
TDR1	 	.equ		H'FFFFBB
SSR1	 	.equ		H'FFFFBC
RDR1		.equ		H'FFFFBD
TE1			.bequ		5,SCR1
RE1			.bequ		6,SCR1
TDRE1		.bequ		7,SSR1
RDRF1		.bequ		6,SSR1

; ---------------------------------------------------------
;	reset vector
			.section	reset0, data, locate=H'00000
			.data.l		ResetEntry

; ---------------------------------------------------------
			.section	rom, code, locate=H'00100
ResetEntry:
			mov.l		#H'FFFF10, ER7		; set up StackPointer

; ---------------------------------------------------------
;	SRAM initialize
			mov.b		#H'FF, R0L			; set output mode to Address out port
			mov.b		R0L, @H'FFFFC0		; 	port1 (A7-A0)
			mov.b		R0L, @H'FFFFC1		;	port2 (A15-A8)
			mov.b		R0L, @H'FFFFC8		;	port5 (A19-A16)
			mov.b		#H'E8, R0L
			mov.b		R0L, @H'FFFFCD		;	port8 (/CS1)

; ---------------------------------------------------------
;	LCD initialize
			mov.b		#H'FF, R0L			; set output mode to LCD port
			mov.b		R0L, @LcdDir

			jsr			@Wait4ms
			jsr			@Wait4ms
			jsr			@Wait4ms
			jsr			@Wait4ms

			mov.b		#B'00100011, R0L	; E=1, RS=0, D=03H function set (8bit)
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN
			jsr			@Wait4ms

			mov.b		#B'00100011, R0L	; E=1, RS=0, D=03H function set (8bit)
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN
			jsr			@Wait4ms

			mov.b		#B'00100011, R0L	; E=1, RS=0, D=03H function set (8bit)
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN
			jsr			@Wait4ms

			mov.b		#B'00100010, R0L	; E=1, RS=0, D=02H interface 4bit data length
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN
			jsr			@Wait4ms
			mov.b		#B'00101000, R0L	; function set, DL=4bit, F=5x7dot matrix
			jsr			@LcdCommand
			jsr			@Wait4ms
			mov.b		#B'00000110, R0L	; entry mode set, I/D=increment, S=shift off
			jsr			@LcdCommand
			jsr			@Wait4ms
			mov.b		#B'00001000, R0L	; display on/off, D=display off, C=cursor off, B=blink off
			jsr			@LcdCommand
			jsr			@Wait4ms
			mov.b		#B'00000001, R0L	; display clear
			jsr			@LcdCommand
			jsr			@Wait4ms
			mov.b		#B'00001100, R0L	; display on/off, D=display on, C=cursor off, B=blink off
			jsr			@LcdCommand
			jsr			@Wait4ms

; ---------------------------------------------------------
;	RS-232C port initialize
			mov.b		#H'00, R0L
			mov.b		R0L, @SCR1			; 荞ݖgp
			mov.b		#H'00, R0L
			mov.b		R0L, @SMR1			; , 8bit, p, stop1bit, MP֎~, clk
			mov.b		#51, R0L			; 9600bps @ 16MHz
			mov.b		R0L, @BRR1
			mov.b		#H'30, R0L
			mov.b		R0L, @SCR1			; TE, RE = 1

; ---------------------------------------------------------
;	RS-232C send
			mov.b		#H'80, R0L			; locate( 0, 0 )
			mov.l		#LcdMsg1, ER1
			jsr			@LcdPuts

			mov.l		#BootMessage, ER1
			jsr			@SciPuts
			mov.l		#LcdMsg2, ER1
			jsr			@LcdPuts
SciTestLoopLine:
			mov.b		#H'C0, R0L			; locate( 0, 1 )
			jsr			@LcdCommand

			mov.b		#20, R0H
SciTestLoopChar:
			jsr			@SciGetc
			bcs			SciTestLoopChar:8
			jsr			@SciPutc
			jsr			@LcdPutc
			dec.b		R0H
			bne			SciTestLoopChar:8

			bra			SciTestLoopLine:8

; ---------------------------------------------------------
;	Wait 4 millisecond
Wait4ms:
			push.l		ER0
			mov.l		#H'2000, ER0
Wait4msLoop:
			sub.l		#1, ER0
			bne			Wait4msLoop
			pop.l		ER0
			rts

; ---------------------------------------------------------
;	Wait LCD
WaitLcd:
			push.l		ER0
			mov.l		#170, ER0
WaitLcdLoop:
			sub.l		#1, ER0
			bne			Wait4msLoop
			pop.l		ER0
			rts

; ---------------------------------------------------------
;	Send command to LCD controller
;	input:
;		R0L ... send command byte
;
LcdCommand:
			push.l		ER0
;	high 4bit
			mov.b		R0L, R0H			; backup R0L
			shlr.b		R0L					; shift right 4bit
			shlr.b		R0L
			shlr.b		R0L
			shlr.b		R0L
			and.b		#B'00001111, R0L	; E=1, RS=0
			or.b		#B'00100000, R0L
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN				; E=0
			jsr			@WaitLcd
;	low 4bit
			and.b		#B'00001111, R0H	; E=1, RS=0
			or.b		#B'00100000, R0H
			mov.b		R0H, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN				; E=0
			jsr			@WaitLcd
			pop.l		ER0
			rts

; ---------------------------------------------------------
;	Send character to LCD controller
;	input:
;		R0L ... send character byte
;
LcdPutc:
			push.l		ER0
;	high 4bit
			mov.b		R0L, R0H			; backup R0L
			shlr.b		R0L					; shift right 4bit
			shlr.b		R0L
			shlr.b		R0L
			shlr.b		R0L
			and.b		#B'00001111, R0L	; E=1, RS=1
			or.b		#B'00110000, R0L
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN				; E=0
			jsr			@WaitLcd
;	low 4bit
			and.b		#B'00001111, R0H	; E=1, RS=1
			or.b		#B'00110000, R0H
			mov.b		R0H, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN				; E=0
			jsr			@WaitLcd
			pop.l		ER0
			rts

; ---------------------------------------------------------
;	Send string to LCD controller
;	input:
;		ER1 ... address of string
LcdPuts:
			push.l		ER0
			push.l		ER1
LcdPutsLoop:
			mov.b		@ER1, R0L
			beq			LcdPutsLoopExit
			jsr			@LcdPutc
			add.l		#1, ER1
			bra			LcdPutsLoop
LcdPutsLoopExit:
			pop.l		ER1
			pop.l		ER0
			rts

; ---------------------------------------------------------
;	Send character to RS-232C port
SciPutc:
			push.w		R1
SciPutcWait:
			mov.b		@SSR1, R1L		; check TDRE
			bpl			SciPutcWait		; if TDRE=0 then go to SciPutc
			mov.b		R0L, @TDR1		; send data
			bclr		#7,R1L			; clear TDRE
			mov.b		R1L, @SSR1
			pop.w		R1
			rts

; ---------------------------------------------------------
;	Receive character from RS-232C port
SciGetc:
			push.w		R1
SciGetcWait:
			mov.b		@SSR1, R1L
			and.b		#H'78, R1L
			beq			SciGetcWait:8	; RDRF, ORER, FER, PER = 0
			bld			#6,R1L			; check RDRF
			bcc			SciGetcError:8	; if RDRF=0 then go to SciGetcError
			mov.b		@RDR1, R0L		; read RDR
			mov.b		#H'84, R1L
			mov.b		R1L, @SSR1		; clear error flag
			pop.w		R1
			andc.b		#H'FE, CCR		; C = 0 (no error)
			rts
SciGetcError:
			and.b		#H'84, R1L
			mov.b		R1L,@SSR1		; clear error flag
			pop.w		R1
			orc.b		#1, CCR			; C = 1 (error)
			rts

; ---------------------------------------------------------
;	Send string to RS-232C
;	input:
;		ER1 ... address of string
SciPuts:
			push.l		ER0
			push.l		ER1
SciPutsLoop:
			mov.b		@ER1, R0L
			beq			SciPutsLoopExit
			jsr			@SciPutc
			add.l		#1, ER1
			bra			SciPutsLoop
SciPutsLoopExit:
			pop.l		ER1
			pop.l		ER0
			rts

; ---------------------------------------------------------
;	Hex code dump
;	input:
;		ER1 ... hex code
DumpHex:
			push.l		ER0
			push.l		ER1
			mov.b		#8, R0H
DumpHexLoop:
			rotl.l		ER1
			rotl.l		ER1
			rotl.l		ER1
			rotl.l		ER1
			push.l		ER1
			and.l		#H'0000000F, ER1
			add.l		#HexChar, ER1
			mov.b		@ER1, R0L
			jsr			@LcdPutc
			pop.l		ER1
			dec			R0H
			bne			DumpHexLoop
			pop.l		ER1
			pop.l		ER0
			rts

; ---------------------------------------------------------
;	Resources
;
BootMessage:
			.sdata		"Boot loader for H8/3048F"
			.data.b		13, 10
			.sdata		"(C)2004 t.hara"
			.data.b		13, 10, 0
LcdMsg1:
			.sdata		"Boot "
			.data.b		0
LcdMsg2:
			.sdata		"Ok"
			.data.b		0

HexChar:	.sdata		"0123456789ABCDEF"
			.end
